require 'json'
require 'sketchup.rb'
require 'fileutils'

module JotunPlugins
  def self.file_separator
    if RUBY_PLATFORM.include?('darwin')
      return File::SEPARATOR
    else
      return File::ALT_SEPARATOR
    end
  end
  def self.get_jotun_materials(materials = Sketchup.active_model.materials)
    response = {
      "path" => "Jotun",
      "surfaces" => [],
      "folders" =>
      materials.select { |m| !m.attribute_dictionaries.nil? && !m.attribute_dictionaries["JotunPlugin"].nil? }
        .sort_by { |m| m.get_attribute("JotunPlugin", "path") }
        .chunk_while do |a, b|
          a.get_attribute("JotunPlugin", "path").start_with?(b.get_attribute("JotunPlugin", "path")[0, a.get_attribute("JotunPlugin", "path").length])
        end.to_a
        .map { |ms|
          {
            "path" => ms[0].get_attribute("JotunPlugin", "path"),
            "surfaces" => ms.map { |m|
              {
                "name" => m.display_name,
                "path" => m.get_attribute("JotunPlugin", "path"),
                "id" => m.get_attribute("JotunPlugin", "id"),
                "color" => {"red"=> m.color.red, "green" => m.color.green, "blue" => m.color.blue},
                "product" => {
                  "ambient" => 0.2,
                  "diffuse" => 0.8,
                  "specular" => 0.0,
                  "shine" => 0.0,
                },
              }
            },
            "folders" => []
          }
        }
    }
    return response
  end
  class JotunMaterialObserver < Sketchup::MaterialsObserver
    def initialize(dialog)
      @dialog = dialog
    end
    def onMaterialAdd(materials, material)
    end
    def onMaterialChange(materials, material)
      jotunMaterials = JotunPlugins.get_jotun_materials()
      @dialog.execute_script("JotunEvents.onMaterialsModified('#{jotunMaterials.to_json}')")
    end
    def onMaterialRemove(materials, material)
      jotunMaterials = JotunPlugins.get_jotun_materials()
      @dialog.execute_script("JotunEvents.onMaterialsModified('#{jotunMaterials.to_json}')")
    end
  end
  class JotunSelectionObserver < Sketchup::SelectionObserver
    def initialize(dialog)
      @dialog = dialog
    end
    # onSelectionChange is not called when only 1 item is selected
    def onSelectionBulkChange(selection)
      @dialog.execute_script("JotunEvents.onFaceSelected(#{selection.first.persistent_id})")
    end
  end

  module JotunPlugin
    def self.onIsPaintingChanged(isPainting)
      @isPainting = isPainting
      @htmlDialog.execute_script("JotunEvents.onIsPainting(#{@isPainting})")
    end
    def self.open_palette
      @htmlDialog = UI::HtmlDialog.new(
        {
          :dialog_title => "Jotun Color Palette",
          :style => UI::HtmlDialog::STYLE_UTILITY,
        })

      if RUBY_PLATFORM.include?('darwin')
        @htmlDialog.set_url("file://#{Sketchup.find_support_file('Plugins')}/jotun/web/index.html")
      else
        @htmlDialog.set_url("file:\\\\#{Sketchup.find_support_file('Plugins')}\\jotun\\web\\index.html")
      end

      @materialObserver = JotunMaterialObserver.new(@htmlDialog)
      Sketchup.active_model.materials.add_observer(@materialObserver)

      @htmlDialog.add_action_callback("resize") { |action_context, param|
        data = JSON.parse(param)
        width = data["width"]
        height = data["height"]
        @htmlDialog.set_content_size(width, height)
      }

      @htmlDialog.add_action_callback("hide") { |action_context, param|
        # Hide is close
        @htmlDialog.close()
      }

      @htmlDialog.add_action_callback("createMaterials") { |action_context, param|
        data = JSON.parse(param)
        materials = Sketchup.active_model.materials
        data.each do |value|
          if materials[value["name"]] != nil
            next
          end
          material = materials.add(value["name"])
          material.set_attribute("JotunPlugin", "path", value["path"])
          material.set_attribute("JotunPlugin", "id", value["name"])
          material.color = [value["color"]["red"], value["color"]["green"], value["color"]["blue"]]

        end
      }
      @htmlDialog.add_action_callback("beginPaint") { |ctx, param|
        if @selectionObserver != nil
          Sketchup.active_model.selection.remove_observer(@selectionObserver)
        end
        @selectionObserver = JotunSelectionObserver.new(@htmlDialog)
        Sketchup.active_model.selection.add_observer(@selectionObserver)
        self.onIsPaintingChanged(true)
        Sketchup.focus()
      }

      @htmlDialog.add_action_callback("deleteMaterial") {  |ctx, param|
        materials = Sketchup.active_model.materials
        materials.remove(param)
      }

      @htmlDialog.add_action_callback("getMaterials") { |ctx, param|
        jotunMaterials = JotunPlugins.get_jotun_materials()
        @htmlDialog.execute_script("JotunEvents.onMaterialsModified('#{jotunMaterials.to_json}')")
      }

      @htmlDialog.add_action_callback("paintElement") { |ctx, faceId, materialId|
        materials = Sketchup.active_model.materials
        material = materials[materialId]
        Sketchup.active_model.start_operation("Paint Jotun Color", true)
        Sketchup.active_model.active_entities.find { |e| e.persistent_id == faceId }.material = material
        Sketchup.active_model.commit_operation
      }

      @htmlDialog.add_action_callback("endPaint") { |ctx, param|
        if @selectionObserver != nil
          Sketchup.active_model.selection.remove_observer(@selectionObserver)
        end
        self.onIsPaintingChanged(false)
      }

      @htmlDialog.show
      @htmlDialog.execute_script("JotunEvents.onCreated()")
      @htmlDialog.execute_script("JotunEvents.onShow()")
    end
    unless file_loaded?(__FILE__)
      menu = UI.menu('Plugins')
      @isPainting = false
      menu.add_item('Jotun Color Palette') { self.open_palette }
      file_loaded(__FILE__)
    end
  end
end


